@@ -25,8 +25,13 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <glib.h>
+
+#include <telepathy-glib/debug.h>
+#include <telepathy-glib/debug-sender.h>
+
 #include "mcd-service.h"
 
+static TpDebugSender *debug_sender;
 
 static void
 on_abort (McdService * mcd)
@@ -46,7 +51,21 @@ main (int argc, char **argv)
     McdService *mcd;
 
     g_type_init ();
+    g_set_application_name ("Account manager");
 
+    /* Keep a ref to the default TpDebugSender for the lifetime of the
+     * McdMaster, so it will persist for the lifetime of MC, and subsequent
+     * calls to tp_debug_sender_dup() will return it again */
+    debug_sender = tp_debug_sender_dup ();
+
+    /* Send all debug messages through the Telepathy infrastructure.
+     *
+     * Unlike CMs, we don't have "subdomains" within MC yet, so we don't want
+     * to exclude any domains. */
+    g_log_set_default_handler (tp_debug_sender_log_handler, NULL);
+
+    mcd_debug_init ();
+    tp_debug_set_flags (g_getenv ("MC_TP_DEBUG"));
 
     mcd = mcd_service_new ();
 
@@ -58,5 +77,7 @@ main (int argc, char **argv)
 
     mcd_service_run (MCD_OBJECT (mcd));
 
+    g_object_unref (debug_sender);
+
     return 0;
 }
