@@ -38,6 +38,7 @@
 #define _POSIX_C_SOURCE 200112L  /* for strtok_r() */
 #include "config.h"
 #include "mcd-manager.h"
+#include "mcd-manager-priv.h"
 #include "mcd-misc.h"
 
 #include <stdio.h>
@@ -473,6 +474,23 @@ mcd_manager_get_parameters (McdManager *manager, const gchar *protocol)
     return NULL;
 }
 
+TpConnectionManagerProtocol *
+_mcd_manager_dup_protocol (McdManager *manager,
+                           const gchar *protocol)
+{
+    const TpConnectionManagerProtocol *p;
+    g_return_val_if_fail (MCD_IS_MANAGER (manager), NULL);
+    g_return_val_if_fail (protocol != NULL, NULL);
+
+    p = tp_connection_manager_get_protocol (manager->priv->tp_conn_mgr,
+                                            protocol);
+
+    if (p == NULL)
+        return NULL;
+    else
+        return tp_connection_manager_protocol_copy (p);
+}
+
 const TpConnectionManagerParam *
 mcd_manager_get_protocol_param (McdManager *manager, const gchar *protocol,
                                 const gchar *param)
